#Python3
#Author: Diana Montes-Grajales (16/02/2021)

import os
import pandas as pd
import numpy as np
import xlsxwriter

#glob.glob("./*.xlsx")

#Generate a dataframe (df) with the data of the excel file.
# The excel file can be downloaded from: https://drive.google.com/drive/folders/1HJjcQScB6DltotBzjSNHCUwTX5lE_Yol?usp=sharing

df= pd.read_excel("./MHC_II_data.xlsx")


####Define a custom funtion that calculates weighted averages

def WAF(group, Frequency, Sample_size):
    f = group[Frequency]
    s = group[Sample_size]
    try:
        return (f * s).sum() / s.sum()          
    except ZeroDivisionError:
        return d.mean()
    
#Calculate weighted allele frequencies (WAF_ALL) and sample sizes (SS_All) for the whole dataset. 

WAF_All=pd.DataFrame(df.groupby("Reported_Alleles").apply(WAF,"Frequency","Sample_size"))
SS_All=pd.DataFrame(df.groupby("Reported_Alleles")["Sample_size"].sum())

#Calculate weighted allele frequencies (WAF_Ethnicity) and sample sizes (SS_Ethnicity) for each group.
WAF_Ethnicity = pd.DataFrame(df.groupby(["Ethnicity","Reported_Alleles"], as_index=False).apply(WAF,"Frequency","Sample_size").pivot("Reported_Alleles", "Ethnicity").fillna(0))
SS_Ethnicity= pd.DataFrame(df.groupby(["Ethnicity","Reported_Alleles"],as_index=False)["Sample_size"].sum().pivot("Reported_Alleles", "Ethnicity").fillna(0))

#Merge results for all (WAF_All and SS_All) and grouped values (WAF_Ethnicity and SS_Ethnicity).
WAF_complete= pd.merge(WAF_All, WAF_Ethnicity, on="Reported_Alleles")
SS_complete= pd.merge(SS_All, SS_Ethnicity, on="Reported_Alleles")

Table_Complete= pd.merge(WAF_complete, SS_complete, on="Reported_Alleles",suffixes=["_WAF", "_SS"])

#Configure column names and organize them.
Table_Complete.columns=["All (Frequency %)","African Colombians (Frequency %)","Colombian Amerindians (Frequency %)","Mestizo (Frequency %)", "All (Sample Size)","African Colombians (Sample Size)","Colombian Amerindians (Sample Size)","Mestizo (Sample Size)"]
Table_Complete= Table_Complete[["All (Frequency %)", "All (Sample Size)","Mestizo (Frequency %)", "Mestizo (Sample Size)","African Colombians (Frequency %)", "African Colombians (Sample Size)", "Colombian Amerindians (Frequency %)", "Colombian Amerindians (Sample Size)"]]
Table_Complete= Table_Complete.sort_values(by=["All (Frequency %)", "Mestizo (Frequency %)","African Colombians (Frequency %)","Colombian Amerindians (Frequency %)"],ascending=False)
print (Table_Complete)

Table_Complete.to_excel ("WAF_HLA_Class_II_Colombia.xlsx", index = True, header=True)
                             
